/* This is a generated file, edit the .stub.php files instead. */

static const func_info_t func_infos[] = {
	F1("zend_version", MAY_BE_STRING),
	FN("func_get_args", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ANY),
	F1("get_class_vars", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("get_class_methods", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("get_included_files", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	FN("set_error_handler", MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_OBJECT|MAY_BE_OBJECT|MAY_BE_NULL),
	FN("set_exception_handler", MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_OBJECT|MAY_BE_OBJECT|MAY_BE_NULL),
	F1("get_declared_classes", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("get_declared_traits", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("get_declared_interfaces", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("get_defined_functions", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ARRAY),
	F1("get_defined_vars", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("get_resource_type", MAY_BE_STRING),
	F1("get_loaded_extensions", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("get_defined_constants", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY),
	F1("debug_backtrace", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ARRAY),
	F1("get_extension_funcs", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("gc_status", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_DOUBLE|MAY_BE_ARRAY_OF_FALSE|MAY_BE_ARRAY_OF_TRUE),
	F1("bcadd", MAY_BE_STRING),
	F1("bcsub", MAY_BE_STRING),
	F1("bcmul", MAY_BE_STRING),
	F1("bcdiv", MAY_BE_STRING),
	F1("bcmod", MAY_BE_STRING),
	F1("bcpowmod", MAY_BE_STRING),
	F1("bcpow", MAY_BE_STRING),
	F1("bcsqrt", MAY_BE_STRING),
	FN("bzopen", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("bzerror", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("cal_from_jd", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_NULL),
	F1("cal_info", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY),
	F1("curl_copy_handle", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("curl_error", MAY_BE_STRING),
	F1("curl_escape", MAY_BE_STRING|MAY_BE_FALSE),
	F1("curl_unescape", MAY_BE_STRING|MAY_BE_FALSE),
	F1("curl_file_create", MAY_BE_OBJECT),
	F1("curl_init", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("curl_multi_info_read", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_OBJECT|MAY_BE_FALSE),
	F1("curl_multi_init", MAY_BE_OBJECT),
	F1("curl_multi_strerror", MAY_BE_STRING|MAY_BE_NULL),
	F1("curl_share_init", MAY_BE_OBJECT),
	F1("curl_share_strerror", MAY_BE_STRING|MAY_BE_NULL),
	F1("curl_strerror", MAY_BE_STRING|MAY_BE_NULL),
	F1("curl_version", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("date", MAY_BE_STRING),
	F1("gmdate", MAY_BE_STRING),
	F1("strftime", MAY_BE_STRING|MAY_BE_FALSE),
	F1("gmstrftime", MAY_BE_STRING|MAY_BE_FALSE),
	F1("localtime", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG),
	F1("getdate", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("date_create", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("date_create_immutable", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("date_create_from_format", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("date_create_immutable_from_format", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("date_parse", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY),
	F1("date_parse_from_format", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY),
	F1("date_get_last_errors", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("date_format", MAY_BE_STRING),
	F1("date_timezone_get", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("date_diff", MAY_BE_OBJECT),
	F1("timezone_open", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("timezone_name_get", MAY_BE_STRING),
	F1("timezone_name_from_abbr", MAY_BE_STRING|MAY_BE_FALSE),
	F1("timezone_transitions_get", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("timezone_location_get", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_DOUBLE|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("timezone_identifiers_list", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("timezone_abbreviations_list", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ARRAY),
	F1("timezone_version_get", MAY_BE_STRING),
	F1("date_interval_create_from_date_string", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("date_interval_format", MAY_BE_STRING),
	F1("date_default_timezone_get", MAY_BE_STRING),
	F1("date_sunrise", MAY_BE_STRING|MAY_BE_LONG|MAY_BE_DOUBLE|MAY_BE_FALSE),
	F1("date_sunset", MAY_BE_STRING|MAY_BE_LONG|MAY_BE_DOUBLE|MAY_BE_FALSE),
	F1("date_sun_info", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_FALSE|MAY_BE_ARRAY_OF_TRUE|MAY_BE_ARRAY_OF_LONG),
	FN("dba_popen", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("dba_open", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("dba_key_split", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("dba_handlers", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING),
	FN("dba_list", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("enchant_broker_list_dicts", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ARRAY),
	F1("enchant_broker_describe", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ARRAY),
	F1("enchant_dict_suggest", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("enchant_dict_describe", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING),
	F1("exif_tagname", MAY_BE_STRING|MAY_BE_FALSE),
	F1("exif_read_data", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_FALSE),
	F1("exif_thumbnail", MAY_BE_STRING|MAY_BE_FALSE),
	F1("finfo_open", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("finfo_file", MAY_BE_STRING|MAY_BE_FALSE),
	F1("finfo_buffer", MAY_BE_STRING|MAY_BE_FALSE),
	F1("mime_content_type", MAY_BE_STRING|MAY_BE_FALSE),
	F1("filter_input_array", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF|MAY_BE_FALSE|MAY_BE_NULL),
	F1("filter_var_array", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF|MAY_BE_FALSE|MAY_BE_NULL),
	F1("filter_list", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("ftp_raw", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_NULL),
	F1("ftp_nlist", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("ftp_rawlist", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("ftp_mlsd", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("gd_info", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_FALSE|MAY_BE_ARRAY_OF_TRUE),
	F1("imagecreatetruecolor", MAY_BE_OBJECT|MAY_BE_FALSE),
#if defined(PHP_WIN32)
	F1("imagegrabwindow", MAY_BE_OBJECT|MAY_BE_FALSE),
#endif
#if defined(PHP_WIN32)
	F1("imagegrabscreen", MAY_BE_OBJECT|MAY_BE_FALSE),
#endif
	F1("imagerotate", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("imagecreate", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("imagecreatefromstring", MAY_BE_OBJECT|MAY_BE_FALSE),
#if defined(HAVE_GD_AVIF)
	F1("imagecreatefromavif", MAY_BE_OBJECT|MAY_BE_FALSE),
#endif
	F1("imagecreatefromgif", MAY_BE_OBJECT|MAY_BE_FALSE),
#if defined(HAVE_GD_JPG)
	F1("imagecreatefromjpeg", MAY_BE_OBJECT|MAY_BE_FALSE),
#endif
#if defined(HAVE_GD_PNG)
	F1("imagecreatefrompng", MAY_BE_OBJECT|MAY_BE_FALSE),
#endif
#if defined(HAVE_GD_WEBP)
	F1("imagecreatefromwebp", MAY_BE_OBJECT|MAY_BE_FALSE),
#endif
	F1("imagecreatefromxbm", MAY_BE_OBJECT|MAY_BE_FALSE),
#if defined(HAVE_GD_XPM)
	F1("imagecreatefromxpm", MAY_BE_OBJECT|MAY_BE_FALSE),
#endif
	F1("imagecreatefromwbmp", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("imagecreatefromgd", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("imagecreatefromgd2", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("imagecreatefromgd2part", MAY_BE_OBJECT|MAY_BE_FALSE),
#if defined(HAVE_GD_BMP)
	F1("imagecreatefrombmp", MAY_BE_OBJECT|MAY_BE_FALSE),
#endif
	F1("imagecolorsforindex", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG),
	F1("imagegetclip", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG),
#if defined(HAVE_GD_FREETYPE)
	F1("imageftbbox", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG|MAY_BE_FALSE),
#endif
#if defined(HAVE_GD_FREETYPE)
	F1("imagefttext", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG|MAY_BE_FALSE),
#endif
	F1("imagecrop", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("imagecropauto", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("imagescale", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("imageaffine", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("imageaffinematrixget", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_DOUBLE|MAY_BE_FALSE),
	F1("imageaffinematrixconcat", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_DOUBLE|MAY_BE_FALSE),
	F1("imageresolution", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG|MAY_BE_TRUE),
	F1("textdomain", MAY_BE_STRING),
	F1("gettext", MAY_BE_STRING),
	F1("dgettext", MAY_BE_STRING),
	F1("dcgettext", MAY_BE_STRING),
	F1("bindtextdomain", MAY_BE_STRING|MAY_BE_FALSE),
#if defined(HAVE_NGETTEXT)
	F1("ngettext", MAY_BE_STRING),
#endif
#if defined(HAVE_DNGETTEXT)
	F1("dngettext", MAY_BE_STRING),
#endif
#if defined(HAVE_DCNGETTEXT)
	F1("dcngettext", MAY_BE_STRING),
#endif
#if defined(HAVE_BIND_TEXTDOMAIN_CODESET)
	F1("bind_textdomain_codeset", MAY_BE_STRING|MAY_BE_FALSE),
#endif
	F1("gmp_div_qr", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_OBJECT),
	F1("gmp_sqrtrem", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_OBJECT),
	F1("gmp_rootrem", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_OBJECT),
	F1("gmp_gcdext", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_OBJECT),
	F1("hash", MAY_BE_STRING),
	F1("hash_file", MAY_BE_STRING|MAY_BE_FALSE),
	F1("hash_hmac", MAY_BE_STRING),
	F1("hash_hmac_file", MAY_BE_STRING|MAY_BE_FALSE),
	F1("hash_init", MAY_BE_OBJECT),
	F1("hash_final", MAY_BE_STRING),
	F1("hash_copy", MAY_BE_OBJECT),
	F1("hash_algos", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("hash_hmac_algos", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("hash_pbkdf2", MAY_BE_STRING),
	F1("hash_hkdf", MAY_BE_STRING),
#if defined(PHP_MHASH_BC)
	F1("mhash_get_hash_name", MAY_BE_STRING|MAY_BE_FALSE),
#endif
#if defined(PHP_MHASH_BC)
	F1("mhash_keygen_s2k", MAY_BE_STRING|MAY_BE_FALSE),
#endif
#if defined(PHP_MHASH_BC)
	F1("mhash", MAY_BE_STRING|MAY_BE_FALSE),
#endif
	F1("iconv_substr", MAY_BE_STRING|MAY_BE_FALSE),
	F1("iconv_mime_encode", MAY_BE_STRING|MAY_BE_FALSE),
	F1("iconv_mime_decode", MAY_BE_STRING|MAY_BE_FALSE),
	F1("iconv_mime_decode_headers", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("iconv", MAY_BE_STRING|MAY_BE_FALSE),
	F1("iconv_get_encoding", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_STRING|MAY_BE_FALSE),
	F1("intlcal_get_available_locales", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("datefmt_localtime", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_FALSE),
	F1("locale_get_keywords", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE|MAY_BE_NULL),
	F1("msgfmt_parse", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_DOUBLE|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("msgfmt_parse_message", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_DOUBLE|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("resourcebundle_locales", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("transliterator_list_ids", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("json_encode", MAY_BE_STRING|MAY_BE_FALSE),
	F1("json_last_error_msg", MAY_BE_STRING),
	F1("ldap_get_entries", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("ldap_get_attributes", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY),
	F1("ldap_get_values_len", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("ldap_explode_dn", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	FN("libxml_get_errors", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_OBJECT),
	F1("mb_language", MAY_BE_STRING|MAY_BE_BOOL),
	F1("mb_internal_encoding", MAY_BE_STRING|MAY_BE_BOOL),
	F1("mb_http_input", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_STRING|MAY_BE_FALSE),
	F1("mb_http_output", MAY_BE_STRING|MAY_BE_BOOL),
	F1("mb_detect_order", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_TRUE),
	F1("mb_substitute_character", MAY_BE_STRING|MAY_BE_LONG|MAY_BE_BOOL),
	F1("mb_preferred_mime_name", MAY_BE_STRING|MAY_BE_FALSE),
	F1("mb_output_handler", MAY_BE_STRING),
	F1("mb_strstr", MAY_BE_STRING|MAY_BE_FALSE),
	F1("mb_strrchr", MAY_BE_STRING|MAY_BE_FALSE),
	F1("mb_stristr", MAY_BE_STRING|MAY_BE_FALSE),
	F1("mb_strrichr", MAY_BE_STRING|MAY_BE_FALSE),
	F1("mb_substr", MAY_BE_STRING),
	F1("mb_strcut", MAY_BE_STRING),
	F1("mb_convert_encoding", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_STRING|MAY_BE_FALSE),
	F1("mb_convert_case", MAY_BE_STRING),
	F1("mb_strtoupper", MAY_BE_STRING),
	F1("mb_strtolower", MAY_BE_STRING),
	F1("mb_detect_encoding", MAY_BE_STRING|MAY_BE_FALSE),
	FN("mb_list_encodings", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("mb_encoding_aliases", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("mb_encode_mimeheader", MAY_BE_STRING),
	F1("mb_decode_mimeheader", MAY_BE_STRING),
	F1("mb_convert_kana", MAY_BE_STRING),
	F1("mb_convert_variables", MAY_BE_STRING|MAY_BE_FALSE),
	F1("mb_encode_numericentity", MAY_BE_STRING),
	F1("mb_decode_numericentity", MAY_BE_STRING),
	F1("mb_get_info", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_STRING|MAY_BE_LONG|MAY_BE_FALSE|MAY_BE_NULL),
#if defined(HAVE_MBREGEX)
	F1("mb_regex_encoding", MAY_BE_STRING|MAY_BE_BOOL),
#endif
#if defined(HAVE_MBREGEX)
	F1("mb_ereg_replace", MAY_BE_STRING|MAY_BE_FALSE|MAY_BE_NULL),
#endif
#if defined(HAVE_MBREGEX)
	F1("mb_eregi_replace", MAY_BE_STRING|MAY_BE_FALSE|MAY_BE_NULL),
#endif
#if defined(HAVE_MBREGEX)
	F1("mb_ereg_replace_callback", MAY_BE_STRING|MAY_BE_FALSE|MAY_BE_NULL),
#endif
#if defined(HAVE_MBREGEX)
	F1("mb_split", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
#endif
#if defined(HAVE_MBREGEX)
	F1("mb_ereg_search_pos", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG|MAY_BE_FALSE),
#endif
#if defined(HAVE_MBREGEX)
	F1("mb_ereg_search_regs", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_FALSE|MAY_BE_FALSE),
#endif
#if defined(HAVE_MBREGEX)
	F1("mb_ereg_search_getregs", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_FALSE|MAY_BE_FALSE),
#endif
#if defined(HAVE_MBREGEX)
	F1("mb_regex_set_options", MAY_BE_STRING),
#endif
	F1("mysqli_affected_rows", MAY_BE_LONG|MAY_BE_STRING),
	F1("mysqli_character_set_name", MAY_BE_STRING),
	F1("mysqli_connect", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("mysqli_connect_error", MAY_BE_STRING|MAY_BE_NULL),
	F1("mysqli_error", MAY_BE_STRING),
	F1("mysqli_error_list", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ARRAY),
	F1("mysqli_fetch_field", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("mysqli_fetch_fields", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_OBJECT),
	F1("mysqli_fetch_field_direct", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("mysqli_fetch_lengths", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG|MAY_BE_FALSE),
	F1("mysqli_fetch_all", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY),
	F1("mysqli_fetch_array", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_NULL|MAY_BE_FALSE),
	F1("mysqli_fetch_assoc", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_NULL|MAY_BE_FALSE),
	F1("mysqli_fetch_object", MAY_BE_OBJECT|MAY_BE_NULL|MAY_BE_FALSE),
	F1("mysqli_fetch_row", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ANY|MAY_BE_NULL|MAY_BE_FALSE),
	F1("mysqli_get_connection_stats", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY),
	F1("mysqli_get_client_stats", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING),
	F1("mysqli_get_charset", MAY_BE_OBJECT|MAY_BE_NULL),
	F1("mysqli_get_client_info", MAY_BE_STRING),
	F1("mysqli_get_links_stats", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG),
	F1("mysqli_get_host_info", MAY_BE_STRING),
	F1("mysqli_get_server_info", MAY_BE_STRING),
	F1("mysqli_get_warnings", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("mysqli_init", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("mysqli_info", MAY_BE_STRING|MAY_BE_NULL),
	F1("mysqli_insert_id", MAY_BE_LONG|MAY_BE_STRING),
	F1("mysqli_num_rows", MAY_BE_LONG|MAY_BE_STRING),
	F1("mysqli_prepare", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("mysqli_query", MAY_BE_OBJECT|MAY_BE_BOOL),
	F1("mysqli_real_escape_string", MAY_BE_STRING),
	F1("mysqli_reap_async_query", MAY_BE_OBJECT|MAY_BE_BOOL),
	F1("mysqli_stmt_affected_rows", MAY_BE_LONG|MAY_BE_STRING),
	F1("mysqli_stmt_error", MAY_BE_STRING),
	F1("mysqli_stmt_error_list", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ARRAY),
	F1("mysqli_stmt_get_result", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("mysqli_stmt_get_warnings", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("mysqli_stmt_init", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("mysqli_stmt_insert_id", MAY_BE_LONG|MAY_BE_STRING),
	F1("mysqli_stmt_num_rows", MAY_BE_LONG|MAY_BE_STRING),
	F1("mysqli_stmt_result_metadata", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("mysqli_stmt_sqlstate", MAY_BE_STRING),
	F1("mysqli_sqlstate", MAY_BE_STRING),
	F1("mysqli_stat", MAY_BE_STRING|MAY_BE_FALSE),
	F1("mysqli_store_result", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("mysqli_use_result", MAY_BE_OBJECT|MAY_BE_FALSE),
	FN("oci_new_connect", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("oci_connect", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("oci_pconnect", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("oci_error", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	FN("oci_parse", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("oci_get_implicit_resultset", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("oci_password_change", MAY_BE_RESOURCE|MAY_BE_BOOL),
	FN("oci_new_cursor", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("odbc_prepare", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("odbc_exec", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("odbc_connect", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("odbc_pconnect", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("odbc_tables", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("odbc_columns", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("odbc_gettypeinfo", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("odbc_primarykeys", MAY_BE_RESOURCE|MAY_BE_FALSE),
#if !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) && !defined(HAVE_SOLID_35)
	FN("odbc_procedurecolumns", MAY_BE_RESOURCE|MAY_BE_FALSE),
#endif
#if !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) && !defined(HAVE_SOLID_35)
	FN("odbc_procedures", MAY_BE_RESOURCE|MAY_BE_FALSE),
#endif
#if !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) && !defined(HAVE_SOLID_35)
	FN("odbc_foreignkeys", MAY_BE_RESOURCE|MAY_BE_FALSE),
#endif
	FN("odbc_specialcolumns", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("odbc_statistics", MAY_BE_RESOURCE|MAY_BE_FALSE),
#if !defined(HAVE_DBMAKER) && !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) &&!defined(HAVE_SOLID_35)
	FN("odbc_tableprivileges", MAY_BE_RESOURCE|MAY_BE_FALSE),
#endif
#if !defined(HAVE_DBMAKER) && !defined(HAVE_SOLID) && !defined(HAVE_SOLID_30) &&!defined(HAVE_SOLID_35)
	FN("odbc_columnprivileges", MAY_BE_RESOURCE|MAY_BE_FALSE),
#endif
	F1("opcache_get_status", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_FALSE),
	F1("opcache_get_configuration", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_FALSE),
	F1("openssl_x509_parse", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("openssl_csr_get_subject", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("openssl_pkey_get_details", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("openssl_get_md_methods", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("openssl_get_cipher_methods", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
#if defined(HAVE_EVP_PKEY_EC)
	F1("openssl_get_curve_names", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
#endif
	F1("openssl_get_cert_locations", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING),
	FN("pcntl_signal_get_handler", MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_OBJECT|MAY_BE_OBJECT|MAY_BE_LONG),
	FN("preg_replace", MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_NULL),
	FN("preg_filter", MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_NULL),
	FN("preg_replace_callback", MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_NULL),
	F1("preg_split", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("preg_grep", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF|MAY_BE_FALSE),
	F1("pdo_drivers", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("pg_dbname", MAY_BE_STRING),
	F1("pg_options", MAY_BE_STRING),
	F1("pg_port", MAY_BE_STRING),
	F1("pg_tty", MAY_BE_STRING),
	F1("pg_host", MAY_BE_STRING),
	F1("pg_version", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_NULL),
	F1("pg_parameter_status", MAY_BE_STRING|MAY_BE_FALSE),
	F1("pg_query", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("pg_query_params", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("pg_prepare", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("pg_execute", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("pg_field_name", MAY_BE_STRING),
	F1("pg_field_type_oid", MAY_BE_STRING|MAY_BE_LONG),
	F1("pg_fetch_result", MAY_BE_STRING|MAY_BE_FALSE|MAY_BE_NULL),
	F1("pg_fetch_row", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_NULL|MAY_BE_FALSE),
	F1("pg_fetch_assoc", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_NULL|MAY_BE_FALSE),
	F1("pg_fetch_array", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_NULL|MAY_BE_FALSE),
	F1("pg_fetch_object", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("pg_fetch_all", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ARRAY),
	F1("pg_fetch_all_columns", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_NULL),
	F1("pg_last_oid", MAY_BE_STRING|MAY_BE_LONG|MAY_BE_FALSE),
	F1("pg_lo_create", MAY_BE_STRING|MAY_BE_LONG|MAY_BE_FALSE),
	F1("pg_lo_open", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("pg_lo_read", MAY_BE_STRING|MAY_BE_FALSE),
	F1("pg_lo_import", MAY_BE_STRING|MAY_BE_LONG|MAY_BE_FALSE),
	F1("pg_copy_to", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("pg_escape_string", MAY_BE_STRING),
	F1("pg_escape_bytea", MAY_BE_STRING),
	F1("pg_unescape_bytea", MAY_BE_STRING),
	F1("pg_escape_literal", MAY_BE_STRING|MAY_BE_FALSE),
	F1("pg_escape_identifier", MAY_BE_STRING|MAY_BE_FALSE),
	F1("pg_result_error", MAY_BE_STRING|MAY_BE_FALSE),
	F1("pg_result_error_field", MAY_BE_STRING|MAY_BE_FALSE|MAY_BE_NULL),
	F1("pg_get_result", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("pg_result_status", MAY_BE_STRING|MAY_BE_LONG),
	F1("pg_get_notify", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("pg_socket", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("pg_meta_data", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("pg_convert", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_FALSE),
	F1("pg_insert", MAY_BE_OBJECT|MAY_BE_STRING|MAY_BE_BOOL),
	F1("pg_update", MAY_BE_STRING|MAY_BE_BOOL),
	F1("pg_delete", MAY_BE_STRING|MAY_BE_BOOL),
	F1("pg_select", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_STRING|MAY_BE_FALSE),
#if defined(HAVE_GETGROUPS)
	F1("posix_getgroups", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG|MAY_BE_FALSE),
#endif
	F1("posix_uname", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("posix_times", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_FALSE),
	F1("posix_getgrnam", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_ARRAY_OF_NULL|MAY_BE_FALSE),
	F1("posix_getgrgid", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_ARRAY_OF_NULL|MAY_BE_FALSE),
	F1("posix_getpwnam", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("posix_getpwuid", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
#if defined(HAVE_GETRLIMIT)
	F1("posix_getrlimit", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
#endif
	F1("pspell_suggest", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("random_bytes", MAY_BE_STRING),
#if defined(HAVE_HISTORY_LIST)
	F1("readline_list_history", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
#endif
	F1("session_name", MAY_BE_STRING|MAY_BE_FALSE),
	F1("session_module_name", MAY_BE_STRING|MAY_BE_FALSE),
	F1("session_save_path", MAY_BE_STRING|MAY_BE_FALSE),
	F1("session_create_id", MAY_BE_STRING|MAY_BE_FALSE),
	F1("session_encode", MAY_BE_STRING|MAY_BE_FALSE),
	F1("session_get_cookie_params", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY),
	F1("session_cache_limiter", MAY_BE_STRING|MAY_BE_FALSE),
	F1("socket_get_option", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_LONG|MAY_BE_FALSE),
	FN("socket_export_stream", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("socket_addrinfo_lookup", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_OBJECT|MAY_BE_FALSE),
	F1("socket_addrinfo_explain", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY),
	FN("sodium_crypto_kx_client_session_keys", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	FN("sodium_crypto_kx_server_session_keys", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
#if defined(crypto_secretstream_xchacha20poly1305_ABYTES)
	FN("sodium_crypto_secretstream_xchacha20poly1305_init_push", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
#endif
#if defined(crypto_secretstream_xchacha20poly1305_ABYTES)
	FN("sodium_crypto_secretstream_xchacha20poly1305_pull", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
#endif
	F1("class_implements", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("class_parents", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("class_uses", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("spl_classes", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING),
	F1("spl_object_hash", MAY_BE_STRING),
	F1("ob_get_flush", MAY_BE_STRING|MAY_BE_FALSE),
	F1("ob_get_clean", MAY_BE_STRING|MAY_BE_FALSE),
	F1("ob_list_handlers", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("ob_get_status", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY),
	F1("compact", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	FN("array_fill", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ANY),
	F1("array_fill_keys", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	FN("array_keys", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING),
	FN("array_values", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_count_values", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG),
	F1("array_column", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_reverse", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_flip", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("array_change_key_case", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_intersect_key", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_intersect_assoc", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_uintersect_assoc", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_diff_key", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_diff_ukey", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_udiff", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_diff_assoc", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_diff_uassoc", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_udiff_assoc", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	F1("array_udiff_uassoc", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_ARRAY_OF_REF),
	FN("array_rand", MAY_BE_LONG|MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("base64_encode", MAY_BE_STRING),
	F1("base64_decode", MAY_BE_STRING|MAY_BE_FALSE),
	F1("long2ip", MAY_BE_STRING|MAY_BE_FALSE),
	F1("getenv", MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("getopt", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_ARRAY_OF_FALSE|MAY_BE_FALSE),
#if defined(HAVE_NANOSLEEP)
	F1("time_nanosleep", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_BOOL),
#endif
	F1("get_current_user", MAY_BE_STRING),
	FN("get_cfg_var", MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("error_get_last", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_NULL),
	F1("highlight_file", MAY_BE_STRING|MAY_BE_BOOL),
	F1("php_strip_whitespace", MAY_BE_STRING),
	F1("highlight_string", MAY_BE_STRING|MAY_BE_BOOL),
	F1("ini_get_all", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_ARRAY_OF_NULL|MAY_BE_FALSE),
	F1("set_include_path", MAY_BE_STRING|MAY_BE_FALSE),
	F1("get_include_path", MAY_BE_STRING|MAY_BE_FALSE),
	F1("print_r", MAY_BE_STRING|MAY_BE_BOOL),
#if defined(HAVE_GETSERVBYPORT)
	F1("getservbyport", MAY_BE_STRING|MAY_BE_FALSE),
#endif
#if defined(HAVE_GETPROTOBYNUMBER)
	F1("getprotobynumber", MAY_BE_STRING|MAY_BE_FALSE),
#endif
	F1("parse_ini_file", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_FALSE|MAY_BE_ARRAY_OF_TRUE|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_DOUBLE|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_ARRAY_OF_NULL|MAY_BE_FALSE),
	F1("parse_ini_string", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_FALSE|MAY_BE_ARRAY_OF_TRUE|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_DOUBLE|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_ARRAY_OF_NULL|MAY_BE_FALSE),
#if ZEND_DEBUG
	F1("config_get_hash", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY),
#endif
#if defined(HAVE_GETLOADAVG)
	F1("sys_getloadavg", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_DOUBLE|MAY_BE_FALSE),
#endif
	F1("get_browser", MAY_BE_OBJECT|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_FALSE),
	F1("crypt", MAY_BE_STRING),
#if defined(HAVE_STRPTIME)
	F1("strptime", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
#endif
#if defined(HAVE_GETHOSTNAME)
	F1("gethostname", MAY_BE_STRING|MAY_BE_FALSE),
#endif
	F1("gethostbyaddr", MAY_BE_STRING|MAY_BE_FALSE),
	F1("gethostbyname", MAY_BE_STRING),
	F1("gethostbynamel", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
#if (defined(PHP_WIN32) || defined(HAVE_DNS_SEARCH_FUNC))
	F1("dns_get_record", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
#endif
	F1("md5", MAY_BE_STRING),
	F1("md5_file", MAY_BE_STRING|MAY_BE_FALSE),
	F1("sha1", MAY_BE_STRING),
	F1("sha1_file", MAY_BE_STRING|MAY_BE_FALSE),
#if defined(HAVE_INET_NTOP)
	F1("inet_ntop", MAY_BE_STRING|MAY_BE_FALSE),
#endif
#if defined(HAVE_INET_PTON)
	F1("inet_pton", MAY_BE_STRING|MAY_BE_FALSE),
#endif
	F1("metaphone", MAY_BE_STRING),
	F1("headers_list", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("htmlspecialchars", MAY_BE_STRING),
	F1("htmlentities", MAY_BE_STRING),
	F1("get_html_translation_table", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING),
	F1("bin2hex", MAY_BE_STRING),
	F1("hex2bin", MAY_BE_STRING|MAY_BE_FALSE),
#if defined(HAVE_NL_LANGINFO)
	F1("nl_langinfo", MAY_BE_STRING|MAY_BE_FALSE),
#endif
	F1("wordwrap", MAY_BE_STRING),
	F1("explode", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("strtok", MAY_BE_STRING|MAY_BE_FALSE),
	F1("basename", MAY_BE_STRING),
	F1("dirname", MAY_BE_STRING),
	F1("pathinfo", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_STRING),
	F1("stristr", MAY_BE_STRING|MAY_BE_FALSE),
	F1("strstr", MAY_BE_STRING|MAY_BE_FALSE),
	F1("strrchr", MAY_BE_STRING|MAY_BE_FALSE),
	F1("chunk_split", MAY_BE_STRING),
	FN("substr_replace", MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING),
	F1("quotemeta", MAY_BE_STRING),
	F1("chr", MAY_BE_STRING),
	F1("ucwords", MAY_BE_STRING),
	F1("strrev", MAY_BE_STRING),
	F1("stripcslashes", MAY_BE_STRING),
	F1("stripslashes", MAY_BE_STRING),
	FN("str_replace", MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING),
	FN("str_ireplace", MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING),
	F1("hebrev", MAY_BE_STRING),
	F1("strip_tags", MAY_BE_STRING),
	F1("str_getcsv", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_NULL),
	F1("str_repeat", MAY_BE_STRING),
	F1("count_chars", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_LONG|MAY_BE_STRING),
	F1("localeconv", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY),
	F1("sscanf", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ANY|MAY_BE_LONG|MAY_BE_NULL),
	F1("str_rot13", MAY_BE_STRING),
	F1("str_shuffle", MAY_BE_STRING),
	F1("str_word_count", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_LONG),
	FN("str_split", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("strpbrk", MAY_BE_STRING|MAY_BE_FALSE),
	F1("utf8_encode", MAY_BE_STRING),
	F1("utf8_decode", MAY_BE_STRING),
	FN("opendir", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("dir", MAY_BE_OBJECT|MAY_BE_FALSE),
	F1("getcwd", MAY_BE_STRING|MAY_BE_FALSE),
	F1("readdir", MAY_BE_STRING|MAY_BE_FALSE),
	F1("scandir", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
#if defined(HAVE_GLOB)
	F1("glob", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
#endif
	F1("exec", MAY_BE_STRING|MAY_BE_FALSE),
	F1("system", MAY_BE_STRING|MAY_BE_FALSE),
	F1("escapeshellcmd", MAY_BE_STRING),
	F1("escapeshellarg", MAY_BE_STRING),
	F1("shell_exec", MAY_BE_STRING|MAY_BE_FALSE|MAY_BE_NULL),
	F1("get_meta_tags", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("popen", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("fgets", MAY_BE_STRING|MAY_BE_FALSE),
	F1("fread", MAY_BE_STRING|MAY_BE_FALSE),
	F1("fopen", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("fscanf", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_ANY|MAY_BE_LONG|MAY_BE_FALSE|MAY_BE_NULL),
	F1("fstat", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_FALSE),
	F1("tempnam", MAY_BE_STRING|MAY_BE_FALSE),
	F1("tmpfile", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("file", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("file_get_contents", MAY_BE_STRING|MAY_BE_FALSE),
	F1("fgetcsv", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_NULL|MAY_BE_FALSE),
	F1("realpath", MAY_BE_STRING|MAY_BE_FALSE),
	F1("sys_get_temp_dir", MAY_BE_STRING),
	F1("filetype", MAY_BE_STRING|MAY_BE_FALSE),
	F1("stat", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_FALSE|MAY_BE_ARRAY_OF_TRUE|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("lstat", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_FALSE|MAY_BE_ARRAY_OF_TRUE|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("realpath_cache_get", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ARRAY),
	F1("sprintf", MAY_BE_STRING),
	F1("vsprintf", MAY_BE_STRING),
	F1("fsockopen", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("pfsockopen", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("http_build_query", MAY_BE_STRING),
	F1("image_type_to_mime_type", MAY_BE_STRING),
	F1("image_type_to_extension", MAY_BE_STRING|MAY_BE_FALSE),
	F1("getimagesize", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("getimagesizefromstring", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("phpversion", MAY_BE_STRING|MAY_BE_FALSE),
	F1("php_sapi_name", MAY_BE_STRING|MAY_BE_FALSE),
	F1("php_uname", MAY_BE_STRING),
	F1("php_ini_scanned_files", MAY_BE_STRING|MAY_BE_FALSE),
	F1("php_ini_loaded_file", MAY_BE_STRING|MAY_BE_FALSE),
	F1("iptcembed", MAY_BE_STRING|MAY_BE_BOOL),
	F1("iptcparse", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
#if (defined(HAVE_SYMLINK) || defined(PHP_WIN32))
	F1("readlink", MAY_BE_STRING|MAY_BE_FALSE),
#endif
	F1("decbin", MAY_BE_STRING),
	F1("decoct", MAY_BE_STRING),
	F1("dechex", MAY_BE_STRING),
	F1("base_convert", MAY_BE_STRING),
	F1("number_format", MAY_BE_STRING),
#if defined(HAVE_GETTIMEOFDAY)
	F1("microtime", MAY_BE_STRING|MAY_BE_DOUBLE),
#endif
#if defined(HAVE_GETTIMEOFDAY)
	F1("gettimeofday", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_DOUBLE),
#endif
#if defined(HAVE_GETRUSAGE)
	F1("getrusage", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_FALSE),
#endif
	F1("pack", MAY_BE_STRING),
	F1("unpack", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY|MAY_BE_FALSE),
	F1("password_get_info", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_ARRAY_OF_NULL),
	F1("password_hash", MAY_BE_STRING),
#if defined(PHP_CAN_SUPPORT_PROC_OPEN)
	F1("proc_open", MAY_BE_RESOURCE|MAY_BE_FALSE),
#endif
#if defined(PHP_CAN_SUPPORT_PROC_OPEN)
	F1("proc_get_status", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_FALSE|MAY_BE_ARRAY_OF_TRUE|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING),
#endif
	F1("quoted_printable_decode", MAY_BE_STRING),
	F1("quoted_printable_encode", MAY_BE_STRING),
	F1("soundex", MAY_BE_STRING),
	F1("stream_context_create", MAY_BE_RESOURCE),
	F1("stream_context_get_params", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY),
	FN("stream_context_get_options", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY),
	FN("stream_context_get_default", MAY_BE_RESOURCE),
	FN("stream_context_set_default", MAY_BE_RESOURCE),
	FN("stream_filter_prepend", MAY_BE_RESOURCE|MAY_BE_FALSE),
	FN("stream_filter_append", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("stream_socket_client", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("stream_socket_server", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("stream_socket_accept", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("stream_socket_recvfrom", MAY_BE_STRING|MAY_BE_FALSE),
#if defined(HAVE_SOCKETPAIR)
	F1("stream_socket_pair", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_RESOURCE|MAY_BE_FALSE),
#endif
	F1("stream_get_contents", MAY_BE_STRING|MAY_BE_FALSE),
	FN("stream_get_meta_data", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_ANY),
	F1("stream_get_line", MAY_BE_STRING|MAY_BE_FALSE),
	F1("stream_resolve_include_path", MAY_BE_STRING|MAY_BE_FALSE),
	F1("stream_get_wrappers", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("stream_get_transports", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
#if defined(HAVE_GETTIMEOFDAY)
	F1("uniqid", MAY_BE_STRING),
#endif
	F1("parse_url", MAY_BE_LONG|MAY_BE_STRING|MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_NULL|MAY_BE_FALSE),
	F1("urlencode", MAY_BE_STRING),
	F1("urldecode", MAY_BE_STRING),
	F1("rawurlencode", MAY_BE_STRING),
	F1("rawurldecode", MAY_BE_STRING),
	F1("get_headers", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_KEY_STRING|MAY_BE_ARRAY_OF_STRING|MAY_BE_ARRAY_OF_ARRAY|MAY_BE_FALSE),
	F1("stream_bucket_make_writeable", MAY_BE_OBJECT|MAY_BE_NULL),
	F1("stream_bucket_new", MAY_BE_OBJECT),
	F1("stream_get_filters", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING),
	F1("convert_uuencode", MAY_BE_STRING),
	F1("convert_uudecode", MAY_BE_STRING|MAY_BE_FALSE),
	F1("var_export", MAY_BE_STRING|MAY_BE_NULL),
	F1("serialize", MAY_BE_STRING),
	F1("xml_error_string", MAY_BE_STRING|MAY_BE_NULL),
	F1("xml_parser_get_option", MAY_BE_STRING|MAY_BE_LONG|MAY_BE_BOOL),
	FN("zend_test_create_throwing_resource", MAY_BE_RESOURCE),
	FN("zip_open", MAY_BE_RESOURCE|MAY_BE_LONG|MAY_BE_FALSE),
	FN("zip_read", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("ob_gzhandler", MAY_BE_STRING|MAY_BE_FALSE),
	F1("zlib_get_coding_type", MAY_BE_STRING|MAY_BE_FALSE),
	F1("gzfile", MAY_BE_ARRAY|MAY_BE_ARRAY_KEY_LONG|MAY_BE_ARRAY_OF_STRING|MAY_BE_FALSE),
	F1("gzopen", MAY_BE_RESOURCE|MAY_BE_FALSE),
	F1("zlib_encode", MAY_BE_STRING|MAY_BE_FALSE),
	F1("zlib_decode", MAY_BE_STRING|MAY_BE_FALSE),
	F1("gzdeflate", MAY_BE_STRING|MAY_BE_FALSE),
	F1("gzencode", MAY_BE_STRING|MAY_BE_FALSE),
	F1("gzcompress", MAY_BE_STRING|MAY_BE_FALSE),
	F1("gzinflate", MAY_BE_STRING|MAY_BE_FALSE),
	F1("gzdecode", MAY_BE_STRING|MAY_BE_FALSE),
	F1("gzuncompress", MAY_BE_STRING|MAY_BE_FALSE),
};
