/* This is a generated file, edit the .stub.php file instead.
 * Stub hash: 83971f2cec8c413d6207382e6ebc4ebf500e805f */

ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(arginfo_class_IntlIterator_current, 0, 0, IS_MIXED, 0)
ZEND_END_ARG_INFO()

#define arginfo_class_IntlIterator_key arginfo_class_IntlIterator_current

ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(arginfo_class_IntlIterator_next, 0, 0, IS_VOID, 0)
ZEND_END_ARG_INFO()

#define arginfo_class_IntlIterator_rewind arginfo_class_IntlIterator_next

ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(arginfo_class_IntlIterator_valid, 0, 0, _IS_BOOL, 0)
ZEND_END_ARG_INFO()


ZEND_METHOD(IntlIterator, current);
ZEND_METHOD(IntlIterator, key);
ZEND_METHOD(IntlIterator, next);
ZEND_METHOD(IntlIterator, rewind);
ZEND_METHOD(IntlIterator, valid);


static const zend_function_entry class_IntlIterator_methods[] = {
	ZEND_ME(IntlIterator, current, arginfo_class_IntlIterator_current, ZEND_ACC_PUBLIC)
	ZEND_ME(IntlIterator, key, arginfo_class_IntlIterator_key, ZEND_ACC_PUBLIC)
	ZEND_ME(IntlIterator, next, arginfo_class_IntlIterator_next, ZEND_ACC_PUBLIC)
	ZEND_ME(IntlIterator, rewind, arginfo_class_IntlIterator_rewind, ZEND_ACC_PUBLIC)
	ZEND_ME(IntlIterator, valid, arginfo_class_IntlIterator_valid, ZEND_ACC_PUBLIC)
	ZEND_FE_END
};

static void register_common_symbols(int module_number)
{
	REGISTER_LONG_CONSTANT("U_USING_FALLBACK_WARNING", U_USING_FALLBACK_WARNING, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_ERROR_WARNING_START", U_ERROR_WARNING_START, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_USING_DEFAULT_WARNING", U_USING_DEFAULT_WARNING, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_SAFECLONE_ALLOCATED_WARNING", U_SAFECLONE_ALLOCATED_WARNING, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_STATE_OLD_WARNING", U_STATE_OLD_WARNING, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_STRING_NOT_TERMINATED_WARNING", U_STRING_NOT_TERMINATED_WARNING, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_SORT_KEY_TOO_SHORT_WARNING", U_SORT_KEY_TOO_SHORT_WARNING, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_AMBIGUOUS_ALIAS_WARNING", U_AMBIGUOUS_ALIAS_WARNING, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_DIFFERENT_UCA_VERSION", U_DIFFERENT_UCA_VERSION, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_ERROR_WARNING_LIMIT", U_ERROR_WARNING_LIMIT, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_ZERO_ERROR", U_ZERO_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_ILLEGAL_ARGUMENT_ERROR", U_ILLEGAL_ARGUMENT_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MISSING_RESOURCE_ERROR", U_MISSING_RESOURCE_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INVALID_FORMAT_ERROR", U_INVALID_FORMAT_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_FILE_ACCESS_ERROR", U_FILE_ACCESS_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INTERNAL_PROGRAM_ERROR", U_INTERNAL_PROGRAM_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MESSAGE_PARSE_ERROR", U_MESSAGE_PARSE_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MEMORY_ALLOCATION_ERROR", U_MEMORY_ALLOCATION_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INDEX_OUTOFBOUNDS_ERROR", U_INDEX_OUTOFBOUNDS_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_PARSE_ERROR", U_PARSE_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INVALID_CHAR_FOUND", U_INVALID_CHAR_FOUND, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_TRUNCATED_CHAR_FOUND", U_TRUNCATED_CHAR_FOUND, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_ILLEGAL_CHAR_FOUND", U_ILLEGAL_CHAR_FOUND, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INVALID_TABLE_FORMAT", U_INVALID_TABLE_FORMAT, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INVALID_TABLE_FILE", U_INVALID_TABLE_FILE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BUFFER_OVERFLOW_ERROR", U_BUFFER_OVERFLOW_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_UNSUPPORTED_ERROR", U_UNSUPPORTED_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_RESOURCE_TYPE_MISMATCH", U_RESOURCE_TYPE_MISMATCH, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_ILLEGAL_ESCAPE_SEQUENCE", U_ILLEGAL_ESCAPE_SEQUENCE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_UNSUPPORTED_ESCAPE_SEQUENCE", U_UNSUPPORTED_ESCAPE_SEQUENCE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_NO_SPACE_AVAILABLE", U_NO_SPACE_AVAILABLE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_CE_NOT_FOUND_ERROR", U_CE_NOT_FOUND_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_PRIMARY_TOO_LONG_ERROR", U_PRIMARY_TOO_LONG_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_STATE_TOO_OLD_ERROR", U_STATE_TOO_OLD_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_TOO_MANY_ALIASES_ERROR", U_TOO_MANY_ALIASES_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_ENUM_OUT_OF_SYNC_ERROR", U_ENUM_OUT_OF_SYNC_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INVARIANT_CONVERSION_ERROR", U_INVARIANT_CONVERSION_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INVALID_STATE_ERROR", U_INVALID_STATE_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_COLLATOR_VERSION_MISMATCH", U_COLLATOR_VERSION_MISMATCH, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_USELESS_COLLATOR_ERROR", U_USELESS_COLLATOR_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_NO_WRITE_PERMISSION", U_NO_WRITE_PERMISSION, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_STANDARD_ERROR_LIMIT", U_STANDARD_ERROR_LIMIT, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BAD_VARIABLE_DEFINITION", U_BAD_VARIABLE_DEFINITION, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_PARSE_ERROR_START", U_PARSE_ERROR_START, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MALFORMED_RULE", U_MALFORMED_RULE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MALFORMED_SET", U_MALFORMED_SET, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MALFORMED_SYMBOL_REFERENCE", U_MALFORMED_SYMBOL_REFERENCE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MALFORMED_UNICODE_ESCAPE", U_MALFORMED_UNICODE_ESCAPE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MALFORMED_VARIABLE_DEFINITION", U_MALFORMED_VARIABLE_DEFINITION, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MALFORMED_VARIABLE_REFERENCE", U_MALFORMED_VARIABLE_REFERENCE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MISMATCHED_SEGMENT_DELIMITERS", U_MISMATCHED_SEGMENT_DELIMITERS, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MISPLACED_ANCHOR_START", U_MISPLACED_ANCHOR_START, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MISPLACED_CURSOR_OFFSET", U_MISPLACED_CURSOR_OFFSET, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MISPLACED_QUANTIFIER", U_MISPLACED_QUANTIFIER, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MISSING_OPERATOR", U_MISSING_OPERATOR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MISSING_SEGMENT_CLOSE", U_MISSING_SEGMENT_CLOSE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MULTIPLE_ANTE_CONTEXTS", U_MULTIPLE_ANTE_CONTEXTS, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MULTIPLE_CURSORS", U_MULTIPLE_CURSORS, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MULTIPLE_POST_CONTEXTS", U_MULTIPLE_POST_CONTEXTS, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_TRAILING_BACKSLASH", U_TRAILING_BACKSLASH, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_UNDEFINED_SEGMENT_REFERENCE", U_UNDEFINED_SEGMENT_REFERENCE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_UNDEFINED_VARIABLE", U_UNDEFINED_VARIABLE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_UNQUOTED_SPECIAL", U_UNQUOTED_SPECIAL, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_UNTERMINATED_QUOTE", U_UNTERMINATED_QUOTE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_RULE_MASK_ERROR", U_RULE_MASK_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MISPLACED_COMPOUND_FILTER", U_MISPLACED_COMPOUND_FILTER, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MULTIPLE_COMPOUND_FILTERS", U_MULTIPLE_COMPOUND_FILTERS, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INVALID_RBT_SYNTAX", U_INVALID_RBT_SYNTAX, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INVALID_PROPERTY_PATTERN", U_INVALID_PROPERTY_PATTERN, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MALFORMED_PRAGMA", U_MALFORMED_PRAGMA, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_UNCLOSED_SEGMENT", U_UNCLOSED_SEGMENT, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_ILLEGAL_CHAR_IN_SEGMENT", U_ILLEGAL_CHAR_IN_SEGMENT, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_VARIABLE_RANGE_EXHAUSTED", U_VARIABLE_RANGE_EXHAUSTED, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_VARIABLE_RANGE_OVERLAP", U_VARIABLE_RANGE_OVERLAP, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_ILLEGAL_CHARACTER", U_ILLEGAL_CHARACTER, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INTERNAL_TRANSLITERATOR_ERROR", U_INTERNAL_TRANSLITERATOR_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INVALID_ID", U_INVALID_ID, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_INVALID_FUNCTION", U_INVALID_FUNCTION, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_PARSE_ERROR_LIMIT", U_PARSE_ERROR_LIMIT, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_UNEXPECTED_TOKEN", U_UNEXPECTED_TOKEN, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_FMT_PARSE_ERROR_START", U_FMT_PARSE_ERROR_START, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MULTIPLE_DECIMAL_SEPARATORS", U_MULTIPLE_DECIMAL_SEPARATORS, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MULTIPLE_DECIMAL_SEPERATORS", U_MULTIPLE_DECIMAL_SEPERATORS, CONST_PERSISTENT | CONST_DEPRECATED);
	REGISTER_LONG_CONSTANT("U_MULTIPLE_EXPONENTIAL_SYMBOLS", U_MULTIPLE_EXPONENTIAL_SYMBOLS, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MALFORMED_EXPONENTIAL_PATTERN", U_MALFORMED_EXPONENTIAL_PATTERN, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MULTIPLE_PERCENT_SYMBOLS", U_MULTIPLE_PERCENT_SYMBOLS, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MULTIPLE_PERMILL_SYMBOLS", U_MULTIPLE_PERMILL_SYMBOLS, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_MULTIPLE_PAD_SPECIFIERS", U_MULTIPLE_PAD_SPECIFIERS, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_PATTERN_SYNTAX_ERROR", U_PATTERN_SYNTAX_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_ILLEGAL_PAD_POSITION", U_ILLEGAL_PAD_POSITION, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_UNMATCHED_BRACES", U_UNMATCHED_BRACES, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_UNSUPPORTED_PROPERTY", U_UNSUPPORTED_PROPERTY, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_UNSUPPORTED_ATTRIBUTE", U_UNSUPPORTED_ATTRIBUTE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_FMT_PARSE_ERROR_LIMIT", U_FMT_PARSE_ERROR_LIMIT, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_INTERNAL_ERROR", U_BRK_INTERNAL_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_ERROR_START", U_BRK_ERROR_START, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_HEX_DIGITS_EXPECTED", U_BRK_HEX_DIGITS_EXPECTED, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_SEMICOLON_EXPECTED", U_BRK_SEMICOLON_EXPECTED, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_RULE_SYNTAX", U_BRK_RULE_SYNTAX, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_UNCLOSED_SET", U_BRK_UNCLOSED_SET, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_ASSIGN_ERROR", U_BRK_ASSIGN_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_VARIABLE_REDFINITION", U_BRK_VARIABLE_REDFINITION, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_MISMATCHED_PAREN", U_BRK_MISMATCHED_PAREN, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_NEW_LINE_IN_QUOTED_STRING", U_BRK_NEW_LINE_IN_QUOTED_STRING, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_UNDEFINED_VARIABLE", U_BRK_UNDEFINED_VARIABLE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_INIT_ERROR", U_BRK_INIT_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_RULE_EMPTY_SET", U_BRK_RULE_EMPTY_SET, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_UNRECOGNIZED_OPTION", U_BRK_UNRECOGNIZED_OPTION, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_MALFORMED_RULE_TAG", U_BRK_MALFORMED_RULE_TAG, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_BRK_ERROR_LIMIT", U_BRK_ERROR_LIMIT, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_INTERNAL_ERROR", U_REGEX_INTERNAL_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_ERROR_START", U_REGEX_ERROR_START, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_RULE_SYNTAX", U_REGEX_RULE_SYNTAX, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_INVALID_STATE", U_REGEX_INVALID_STATE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_BAD_ESCAPE_SEQUENCE", U_REGEX_BAD_ESCAPE_SEQUENCE, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_PROPERTY_SYNTAX", U_REGEX_PROPERTY_SYNTAX, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_UNIMPLEMENTED", U_REGEX_UNIMPLEMENTED, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_MISMATCHED_PAREN", U_REGEX_MISMATCHED_PAREN, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_NUMBER_TOO_BIG", U_REGEX_NUMBER_TOO_BIG, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_BAD_INTERVAL", U_REGEX_BAD_INTERVAL, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_MAX_LT_MIN", U_REGEX_MAX_LT_MIN, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_INVALID_BACK_REF", U_REGEX_INVALID_BACK_REF, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_INVALID_FLAG", U_REGEX_INVALID_FLAG, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_LOOK_BEHIND_LIMIT", U_REGEX_LOOK_BEHIND_LIMIT, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_SET_CONTAINS_STRING", U_REGEX_SET_CONTAINS_STRING, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_REGEX_ERROR_LIMIT", U_REGEX_ERROR_LIMIT, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_IDNA_PROHIBITED_ERROR", U_IDNA_PROHIBITED_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_IDNA_ERROR_START", U_IDNA_ERROR_START, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_IDNA_UNASSIGNED_ERROR", U_IDNA_UNASSIGNED_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_IDNA_CHECK_BIDI_ERROR", U_IDNA_CHECK_BIDI_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_IDNA_STD3_ASCII_RULES_ERROR", U_IDNA_STD3_ASCII_RULES_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_IDNA_ACE_PREFIX_ERROR", U_IDNA_ACE_PREFIX_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_IDNA_VERIFICATION_ERROR", U_IDNA_VERIFICATION_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_IDNA_LABEL_TOO_LONG_ERROR", U_IDNA_LABEL_TOO_LONG_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_IDNA_ZERO_LENGTH_LABEL_ERROR", U_IDNA_ZERO_LENGTH_LABEL_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_IDNA_DOMAIN_NAME_TOO_LONG_ERROR", U_IDNA_DOMAIN_NAME_TOO_LONG_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_IDNA_ERROR_LIMIT", U_IDNA_ERROR_LIMIT, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_STRINGPREP_PROHIBITED_ERROR", U_STRINGPREP_PROHIBITED_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_STRINGPREP_UNASSIGNED_ERROR", U_STRINGPREP_UNASSIGNED_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_STRINGPREP_CHECK_BIDI_ERROR", U_STRINGPREP_CHECK_BIDI_ERROR, CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("U_ERROR_LIMIT", U_ERROR_LIMIT, CONST_PERSISTENT);
}

static zend_class_entry *register_class_IntlIterator(zend_class_entry *class_entry_Iterator)
{
	zend_class_entry ce, *class_entry;

	INIT_CLASS_ENTRY(ce, "IntlIterator", class_IntlIterator_methods);
	class_entry = zend_register_internal_class_ex(&ce, NULL);
	class_entry->ce_flags |= ZEND_ACC_NOT_SERIALIZABLE;
	zend_class_implements(class_entry, 1, class_entry_Iterator);

	return class_entry;
}
