/* This is a generated file, edit the .stub.php file instead.
 * Stub hash: 7927099133bf60cab0464aae8c9de8f7ef2732bd */



static void register_pdo_odbc_symbols(int module_number)
{
	REGISTER_STRING_CONSTANT("PDO_ODBC_TYPE", PDO_ODBC_TYPE, CONST_PERSISTENT);
}
